<?php

namespace AustraliaPost\Extensions\Business\API\Wrapper\Connectors;

use AustraliaPost\Helpers\Logger;
use WP_Error;

abstract class Abstract_Connector {


	const PARTNER_ID = 'WPRUBY-8882';

	/**
	 * @return array
	 */
	abstract protected function getHeaders();
	abstract public function  getAccountNumber();
	abstract protected function  apiUrl();

	/**
	 * @return boolean
	 */
	abstract protected function testMode();

	/**
	 * @param $endpoint
	 * @param $method
	 * @param null $payload
	 *
	 * @return array
	 */
	public function connect($endpoint, $method, $payload = null)
	{
		$headers = $this->getHeaders();
		$arguments = [
			'method'  => strtoupper($method),
			'headers' => $headers,
			'timeout' => 10,
		];

		if ($payload !== null ){
			$arguments['body'] = $payload;
		}

		$api_url = $this->apiUrl();
		Logger::log($api_url . $endpoint);

		$response = wp_remote_request( $api_url . $endpoint, $arguments );

		Logger::log($response);

		return $this->handleResponse($response);
	}

	/**
	 * @param $response
	 *
	 * @return array|string
	 */
	private function handleResponse( $response )
	{
		//error
		if ($response instanceof WP_Error){
			return array($response->get_error_code() => $response->get_error_message());
		}

		$header = $response['headers'];
		if ($header['content-type'] === 'application/pdf') {
			return wp_remote_retrieve_body($response);
		}

		$responseBody = json_decode(wp_remote_retrieve_body($response), true);

		if (isset($responseBody['errors'])) {
			return ['errors' => $responseBody['errors']];
		}

		//success
		return $responseBody;

	}
}
